#include <stdio.h>
#include <glib-object.h>
#include <stdlib.h>

#include "tag.h"

GHashTable *tag_table;

void tag_table_add_empty_key(gchar *tag_name)
{
	if (tag_table == NULL)
		tag_table = g_hash_table_new(g_str_hash, g_str_equal);
	g_hash_table_insert(tag_table, tag_name, NULL);
}

void tag_table_insert_value(gchar *tag_name, Tag *tag)
{
	if (tag_name != NULL)
		g_hash_table_insert(tag_table, tag_name, tag);
}

Tag *tag_table_find_tag_by_name(gchar *tag_name)
{
	gpointer ret_tag;
	
	if (tag_name != NULL)
		ret_tag = g_hash_table_lookup(tag_table, tag_name);
		
	return (Tag *)ret_tag;
}

void tag_table_rewrite_tag(gchar *tag_name, Tag *new_tag)
{
	if (tag_name != NULL)
		g_hash_table_replace(tag_table, tag_name, new_tag);
}

void tag_table_write_int(gchar *tag_name, gint value)
{
	Tag *tag;
	
	if (tag_name != NULL) {
		tag = tag_table_find_tag_by_name(tag_name);
		if (G_VALUE_HOLDS_INT(tag->value))	
			g_value_set_int(tag->value, value);
		else
			g_warning("Value not hold int, write fail\n");
	}
}

void tag_table_write_uint(gchar *tag_name, guint value)
{
	Tag *tag;
	
	if (tag_name != NULL) {
		tag = tag_table_find_tag_by_name(tag_name);
		if (G_VALUE_HOLDS_INT(tag->value))	
			g_value_set_uint(tag->value, value);
		else
			g_warning("Value not hold uint, write fail\n");
	}
}


void tag_table_write_double(gchar *tag_name, gdouble value)
{
	Tag *tag;
	
	if (tag_name != NULL) {
		tag = tag_table_find_tag_by_name(tag_name);
		if (G_VALUE_HOLDS_INT(tag->value))	
			g_value_set_double(tag->value, value);
		else
			g_warning("Value not hold double, write fail\n");
	}
}

gint tag_table_read_int(gchar *tag_name)
{
	Tag *tag;
	gint ret;
	
	if (tag_name != NULL) {
		tag = tag_table_find_tag_by_name(tag_name);
		if (G_VALUE_HOLDS_INT(tag->value))
			ret = g_value_get_int(tag->value);
		else
			g_warning("Value not hold int,, read fail\n");
	}
	
	return ret;
}

guint tag_table_read_uint(gchar *tag_name)
{
	Tag *tag;
	guint ret;
	
	if (tag_name != NULL) {
		tag = tag_table_find_tag_by_name(tag_name);
		if (G_VALUE_HOLDS_UINT(tag->value))
			ret = g_value_get_uint(tag->value);
		else
			g_warning("Value not hold uint, read fail\n");
	}
	
	return ret;
}

gdouble tag_table_read_double(gchar *tag_name)
{
	Tag *tag;
	gdouble ret;
	
	if (tag_name != NULL) {
		tag = tag_table_find_tag_by_name(tag_name);
		if (G_VALUE_HOLDS_UINT(tag->value))
			ret = g_value_get_double(tag->value);
		else
			g_warning("Value not hold double, read fail\n");
	}
	
	return ret;
}

